@@ -11,6 +11,7 @@
 #    under the License.
 
 
+from oslo_log import versionutils
 from oslo_policy import policy
 
 from placement.policies import base
@@ -19,10 +20,19 @@ from placement.policies import base
 PREFIX = 'placement:reshaper:%s'
 RESHAPE = PREFIX % 'reshape'
 
+deprecated_reshape = policy.DeprecatedRule(
+    name=RESHAPE,
+    check_str=base.RULE_ADMIN_API,
+)
+
+DEPRECATED_REASON = """
+The reshape API now supports scoped rule by default.
+"""
+
 rules = [
     policy.DocumentedRuleDefault(
         RESHAPE,
-        base.RULE_ADMIN_API,
+        base.SYSTEM_ADMIN,
         "Reshape Inventory and Allocations.",
         [
             {
@@ -30,7 +40,11 @@ rules = [
                 'path': '/reshaper'
             }
         ],
-        scope_types=['system']),
+        scope_types=['system'],
+        deprecated_rule=deprecated_reshape,
+        deprecated_reason=DEPRECATED_REASON,
+        deprecated_since=versionutils.deprecated.WALLABY,
+    ),
 ]
 
 
