@@ -11,16 +11,18 @@
 #    under the License.
 
 
-import mock
+from unittest import mock
+
 import os_resource_classes as orc
 from oslo_db import exception as db_exc
 from oslo_utils.fixture import uuidsentinel
-import sqlalchemy as sa
 
 from placement.db.sqlalchemy import models
 from placement import exception
+from placement import lib as placement_lib
 from placement.objects import allocation as alloc_obj
 from placement.objects import inventory as inv_obj
+from placement.objects import research_context as res_ctx
 from placement.objects import resource_provider as rp_obj
 from placement.objects import trait as trait_obj
 from placement.objects import usage as usage_obj
@@ -88,111 +90,6 @@ class ResourceProviderTestCase(tb.PlacementDbBaseTestCase):
         self.assertEqual(uuidsentinel.create_p, child.parent_provider_uuid)
         self.assertEqual(uuidsentinel.create_p, child.root_provider_uuid)
 
-    def test_root_provider_population(self):
-        """Simulate an old resource provider record in the database that has no
-        root_provider_uuid set and ensure that when grabbing the resource
-        provider object, the root_provider_uuid field in the table is set to
-        the provider's UUID.
-        """
-        rp_tbl = rp_obj._RP_TBL
-        conn = self.placement_db.get_engine().connect()
-
-        # First, set up a record for an "old-style" resource provider with no
-        # root provider UUID.
-        ins_stmt = rp_tbl.insert().values(
-            id=1,
-            uuid=uuidsentinel.rp1,
-            name='rp-1',
-            root_provider_id=None,
-            parent_provider_id=None,
-            generation=42,
-        )
-        conn.execute(ins_stmt)
-
-        rp = rp_obj.ResourceProvider.get_by_uuid(self.ctx, uuidsentinel.rp1)
-
-        # The ResourceProvider._from_db_object() method should have performed
-        # an online data migration, populating the root_provider_id field
-        # with the value of the id field. Let's check it happened.
-        sel_stmt = sa.select([rp_tbl.c.root_provider_id]).where(
-            rp_tbl.c.id == 1)
-        res = conn.execute(sel_stmt).fetchall()
-        self.assertEqual(1, res[0][0])
-        # Make sure the object root_provider_uuid is set on load
-        self.assertEqual(rp.root_provider_uuid, uuidsentinel.rp1)
-
-    def test_set_root_provider(self):
-        """Simulate old resource provider records in the database that has no
-        root_provider_uuid set and ensure the root_provider_uuid field in the
-        table is set to the provider's ID via set_root_provider_ids().
-        """
-        # First, set up records for "old-style" resource providers with
-        # no root provider UUID.
-        rp_tbl = rp_obj._RP_TBL
-        conn = self.placement_db.get_engine().connect()
-
-        ins_stmt1 = rp_tbl.insert().values(
-            id=1,
-            uuid=uuidsentinel.rp1,
-            name='rp-1',
-            root_provider_id=None,
-            parent_provider_id=None,
-            generation=42,
-        )
-        ins_stmt2 = rp_tbl.insert().values(
-            id=2,
-            uuid=uuidsentinel.rp2,
-            name='rp-2',
-            root_provider_id=None,
-            parent_provider_id=None,
-            generation=42,
-        )
-        conn.execute(ins_stmt1)
-        conn.execute(ins_stmt2)
-
-        # Second, set up records for "new-style" resource providers
-        # in a tree
-        self._create_provider('root_rp')
-        self._create_provider('child_rp', parent=uuidsentinel.root_rp)
-        self._create_provider('grandchild_rp', parent=uuidsentinel.child_rp)
-
-        # Check rp_1 that it has no root provider id
-        sel_stmt = sa.select([rp_tbl.c.root_provider_id]).where(
-            rp_tbl.c.id == 1)
-        res = conn.execute(sel_stmt).fetchall()
-        self.assertIsNone(res[0][0])
-        # Check rp_2 that it has no root provider id
-        sel_stmt = sa.select([rp_tbl.c.root_provider_id]).where(
-            rp_tbl.c.id == 2)
-        res = conn.execute(sel_stmt).fetchall()
-        self.assertIsNone(res[0][0])
-
-        # Run set_root_provider_ids()
-        found, migrated = rp_obj.set_root_provider_ids(self.ctx, batch_size=10)
-        self.assertEqual(2, found)
-        self.assertEqual(2, migrated)
-
-        # Check rp_1 that it has got the root provider id
-        sel_stmt = sa.select([rp_tbl.c.root_provider_id]).where(
-            rp_tbl.c.id == 1)
-        res = conn.execute(sel_stmt).fetchall()
-        self.assertEqual(1, res[0][0])
-        # Check rp_2 that it has got the root provider id
-        sel_stmt = sa.select([rp_tbl.c.root_provider_id]).where(
-            rp_tbl.c.id == 2)
-        res = conn.execute(sel_stmt).fetchall()
-        self.assertEqual(2, res[0][0])
-
-        # Check the new-style providers remains in a tree,
-        # which means the root provider ids are not changed
-        rps = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={
-                'in_tree': uuidsentinel.root_rp,
-            }
-        )
-        self.assertEqual(3, len(rps))
-
     def test_inherit_root_from_parent(self):
         """Tests that if we update an existing provider's parent provider UUID,
         that the root provider UUID of the updated provider is automatically
@@ -417,64 +314,20 @@ class ResourceProviderTestCase(tb.PlacementDbBaseTestCase):
         child_rp.destroy()
         root_rp.destroy()
 
-    def test_get_all_in_tree_old_records(self):
-        """Simulate an old resource provider record in the database that has no
-        root_provider_uuid set and ensure that when selecting all providers in
-        a tree, passing in that old resource provider, that we still get that
-        provider returned.
-        """
-        # Passing a non-existing resource provider UUID should return an empty
-        # list
-        rps = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={
-                'in_tree': uuidsentinel.rp1,
-            }
-        )
-        self.assertEqual([], rps)
-
-        rp_tbl = rp_obj._RP_TBL
-        conn = self.placement_db.get_engine().connect()
-
-        # First, set up a record for an "old-style" resource provider with no
-        # root provider UUID.
-        ins_stmt = rp_tbl.insert().values(
-            id=1,
-            uuid=uuidsentinel.rp1,
-            name='rp-1',
-            root_provider_id=None,
-            parent_provider_id=None,
-            generation=42,
-        )
-        conn.execute(ins_stmt)
-
-        # NOTE(jaypipes): This is just disabling the online data migration that
-        # occurs in _from_db_object() that sets root provider ID to ensure we
-        # don't have any migrations messing with the end result.
-        with mock.patch('placement.objects.resource_provider.'
-                        '_set_root_provider_id'):
-            rps = rp_obj.get_all_by_filters(
-                self.ctx,
-                filters={
-                    'in_tree': uuidsentinel.rp1,
-                }
-            )
-        self.assertEqual(1, len(rps))
-
     def test_has_provider_trees(self):
-        """The has_provider_trees() helper method should return False unless
+        """The _has_provider_trees() helper method should return False unless
         there is a resource provider that is a parent.
         """
-        self.assertFalse(rp_obj.has_provider_trees(self.ctx))
+        self.assertFalse(res_ctx._has_provider_trees(self.ctx))
         self._create_provider('cn')
 
         # No parents yet. Should still be False.
-        self.assertFalse(rp_obj.has_provider_trees(self.ctx))
+        self.assertFalse(res_ctx._has_provider_trees(self.ctx))
 
         self._create_provider('numa0', parent=uuidsentinel.cn)
 
         # OK, now we've got a parent, so should be True
-        self.assertTrue(rp_obj.has_provider_trees(self.ctx))
+        self.assertTrue(res_ctx._has_provider_trees(self.ctx))
 
     def test_destroy_resource_provider(self):
         created_resource_provider = self._create_provider(
@@ -609,13 +462,17 @@ class ResourceProviderTestCase(tb.PlacementDbBaseTestCase):
         an exception.
         """
         rp = self._create_provider('compute-host')
-        self.assertRaises(exception.ResourceClassNotFound,
-                          tb.add_inventory, rp, 'UNKNOWN', 1024,
-                          reserved=15,
-                          min_unit=10,
-                          max_unit=100,
-                          step_size=10,
-                          allocation_ratio=1.0)
+        inv = inv_obj.Inventory(
+            rp._context, resource_provider=rp,
+            resource_class='UNKNOWN',
+            total=1024,
+            reserved=15,
+            min_unit=10,
+            max_unit=100,
+            step_size=10,
+            allocation_ratio=1.0)
+        self.assertRaises(
+            exception.ResourceClassNotFound, rp.add_inventory, inv)
 
     def test_set_inventory_fail_in_use(self):
         """Test attempting to set inventory which would result in removing an
@@ -839,26 +696,33 @@ class ResourceProviderTestCase(tb.PlacementDbBaseTestCase):
 
 
 class ResourceProviderListTestCase(tb.PlacementDbBaseTestCase):
+    def _run_get_all_by_filters(self, expected_rps, filters=None):
+        '''Helper function to validate get_all_by_filters()'''
+        resource_providers = rp_obj.get_all_by_filters(self.ctx,
+                                                       filters=filters)
+        self.assertEqual(len(expected_rps), len(resource_providers))
+        rp_names = set([rp.name for rp in resource_providers])
+        self.assertEqual(set(expected_rps), rp_names)
+        return resource_providers
+
     def test_get_all_by_filters(self):
         for rp_i in ['1', '2']:
-            self._create_provider(
-                'rp_name_' + rp_i,
-                uuid=getattr(uuidsentinel, 'rp_uuid_' + rp_i))
+            self._create_provider('rp_' + rp_i)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx)
-        self.assertEqual(2, len(resource_providers))
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, filters={'name': u'rp_name_1'})
-        self.assertEqual(1, len(resource_providers))
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, filters={'uuid': uuidsentinel.rp_uuid_2})
-        self.assertEqual(1, len(resource_providers))
-        self.assertEqual('rp_name_2', resource_providers[0].name)
+        expected_rps = ['rp_1', 'rp_2']
+        self._run_get_all_by_filters(expected_rps)
+
+        filters = {'name': u'rp_1'}
+        expected_rps = ['rp_1']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
+        filters = {'uuid': uuidsentinel.rp_2}
+        expected_rps = ['rp_2']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
     def test_get_all_by_filters_with_resources(self):
         for rp_i in ['1', '2']:
-            rp = self._create_provider('rp_name_' + rp_i)
+            rp = self._create_provider('rp_' + rp_i)
             tb.add_inventory(rp, orc.VCPU, 2)
             tb.add_inventory(rp, orc.DISK_GB, 1024,
                              reserved=2)
@@ -873,50 +737,52 @@ class ResourceProviderListTestCase(tb.PlacementDbBaseTestCase):
 
         # Both RPs should accept that request given the only current allocation
         # for the first RP is leaving one VCPU
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.VCPU: 1}})
-        self.assertEqual(2, len(resource_providers))
+        filters = {'resources': {orc.VCPU: 1}}
+        expected_rps = ['rp_1', 'rp_2']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
         # Now, when asking for 2 VCPUs, only the second RP should accept that
         # given the current allocation for the first RP
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.VCPU: 2}})
-        self.assertEqual(1, len(resource_providers))
+        filters = {'resources': {orc.VCPU: 2}}
+        expected_rps = ['rp_2']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
         # Adding a second resource request should be okay for the 2nd RP
         # given it has enough disk but we also need to make sure that the
         # first RP is not acceptable because of the VCPU request
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.VCPU: 2, orc.DISK_GB: 1022}})
-        self.assertEqual(1, len(resource_providers))
+        filters = {'resources': {orc.VCPU: 2, orc.DISK_GB: 1022}}
+        expected_rps = ['rp_2']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
         # Now, we are asking for both disk and VCPU resources that all the RPs
         # can't accept (as the 2nd RP is having a reserved size)
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.VCPU: 2, orc.DISK_GB: 1024}})
-        self.assertEqual(0, len(resource_providers))
+        filters = {'resources': {orc.VCPU: 2, orc.DISK_GB: 1024}}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
         # We also want to verify that asking for a specific RP can also be
         # checking the resource usage.
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'name': u'rp_name_1',
-                       'resources': {orc.VCPU: 1}})
-        self.assertEqual(1, len(resource_providers))
+        filters = {'name': u'rp_1', 'resources': {orc.VCPU: 1}}
+        expected_rps = ['rp_1']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
         # Let's verify that the min and max units are checked too
         # Case 1: amount is in between min and max and modulo step_size
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.MEMORY_MB: 2}})
-        self.assertEqual(2, len(resource_providers))
+        filters = {'resources': {orc.MEMORY_MB: 2}}
+        expected_rps = ['rp_1', 'rp_2']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
         # Case 2: amount is less than min_unit
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.MEMORY_MB: 1}})
-        self.assertEqual(0, len(resource_providers))
+        filters = {'resources': {orc.MEMORY_MB: 1}}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
         # Case 3: amount is more than min_unit
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.MEMORY_MB: 5}})
-        self.assertEqual(0, len(resource_providers))
+        filters = {'resources': {orc.MEMORY_MB: 5}}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
         # Case 4: amount is not modulo step_size
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, {'resources': {orc.MEMORY_MB: 3}})
-        self.assertEqual(0, len(resource_providers))
+        filters = {'resources': {orc.MEMORY_MB: 3}}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
     def test_get_all_by_filters_with_resources_not_existing(self):
         self.assertRaises(
@@ -927,41 +793,67 @@ class ResourceProviderListTestCase(tb.PlacementDbBaseTestCase):
     def test_get_all_by_filters_aggregate(self):
         for rp_i in [1, 2, 3, 4]:
             aggs = [uuidsentinel.agg_a, uuidsentinel.agg_b] if rp_i % 2 else []
-            self._create_provider(
-                'rp_name_' + str(rp_i), *aggs,
-                uuid=getattr(uuidsentinel, 'rp_uuid_' + str(rp_i)))
+            self._create_provider('rp_' + str(rp_i), *aggs)
+        for rp_i in [5, 6]:
+            aggs = [uuidsentinel.agg_b, uuidsentinel.agg_c]
+            self._create_provider('rp_' + str(rp_i), *aggs)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx, filters={'member_of': [[uuidsentinel.agg_a]]})
+        # Get rps in "agg_a"
+        filters = {'member_of': [[uuidsentinel.agg_a]]}
+        expected_rps = ['rp_1', 'rp_3']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
-        self.assertEqual(2, len(resource_providers))
-        names = [_rp.name for _rp in resource_providers]
-        self.assertIn('rp_name_1', names)
-        self.assertIn('rp_name_3', names)
-        self.assertNotIn('rp_name_2', names)
-        self.assertNotIn('rp_name_4', names)
+        # Validate rps in "agg_a" or "agg_b"
+        filters = {'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]]}
+        expected_rps = ['rp_1', 'rp_3', 'rp_5', 'rp_6']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]]})
-        self.assertEqual(2, len(resource_providers))
+        # Validate rps in "agg_a" or "agg_b" and named "rp_1"
+        filters = {'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
+                   'name': u'rp_1'}
+        expected_rps = ['rp_1']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
-                     'name': u'rp_name_1'})
-        self.assertEqual(1, len(resource_providers))
+        # Validate rps in "agg_a" or "agg_b" and named "barnabas"
+        filters = {'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
+                   'name': u'barnabas'}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
-                     'name': u'barnabas'})
-        self.assertEqual(0, len(resource_providers))
+        # Validate rps in "agg_1" or "agg_2"
+        filters = {'member_of': [[uuidsentinel.agg_1, uuidsentinel.agg_2]]}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
-        resource_providers = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={'member_of': [[uuidsentinel.agg_1, uuidsentinel.agg_2]]})
-        self.assertEqual(0, len(resource_providers))
+        # Validate rps NOT in "agg_a"
+        filters = {'forbidden_aggs': [uuidsentinel.agg_a]}
+        expected_rps = ['rp_2', 'rp_4', 'rp_5', 'rp_6']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
+        # Validate rps NOT in "agg_1"
+        filters = {'forbidden_aggs': [uuidsentinel.agg_1]}
+        expected_rps = ['rp_1', 'rp_2', 'rp_3', 'rp_4', 'rp_5', 'rp_6']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
+        # Validate rps in "agg_a" or "agg_b" that are not in "agg_1"
+        filters = {'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
+                   'forbidden_aggs': [uuidsentinel.agg_1]}
+        expected_rps = ['rp_1', 'rp_3', 'rp_5', 'rp_6']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
+        # Validate rps in "agg_a" or "agg_b" that are not in "agg_a"
+        # ...which means rps in "agg_b"
+        filters = {'member_of': [[uuidsentinel.agg_a, uuidsentinel.agg_b]],
+                   'forbidden_aggs': [uuidsentinel.agg_a]}
+        expected_rps = ['rp_5', 'rp_6']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
+
+        # Validate rps in both "agg_a" and "agg_b" that are not in "agg_a"
+        # ...which means no rp
+        filters = {'member_of': [[uuidsentinel.agg_a], [uuidsentinel.agg_b]],
+                   'forbidden_aggs': [uuidsentinel.agg_a]}
+        expected_rps = []
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
     def test_get_all_by_required(self):
         # Create some resource providers and give them each 0 or more traits.
@@ -972,29 +864,24 @@ class ResourceProviderListTestCase(tb.PlacementDbBaseTestCase):
         trait_names = ['CUSTOM_TRAIT_A', 'CUSTOM_TRAIT_B',
                        'CUSTOM_TRAIT_C']
         for rp_i in [0, 1, 2, 3]:
-            rp = self._create_provider(
-                'rp_name_' + str(rp_i),
-                uuid=getattr(uuidsentinel, 'rp_uuid_' + str(rp_i)))
+            rp = self._create_provider('rp_' + str(rp_i))
             if rp_i:
                 traits = trait_names[0:rp_i]
                 tb.set_traits(rp, *traits)
 
         # Three rps (1, 2, 3) should have CUSTOM_TRAIT_A
-        custom_a_rps = rp_obj.get_all_by_filters(
-            self.ctx, filters={'required': ['CUSTOM_TRAIT_A']})
-        self.assertEqual(3, len(custom_a_rps))
-        rp_names = [a_rp.name for a_rp in custom_a_rps]
-        expected_names = ['rp_name_%s' % i for i in [1, 2, 3]]
-        self.assertEqual(expected_names, sorted(rp_names))
+        filters = {'required': ['CUSTOM_TRAIT_A']}
+        expected_rps = ['rp_1', 'rp_2', 'rp_3']
+        self._run_get_all_by_filters(expected_rps, filters=filters)
 
         # One rp (rp 1) if we forbid CUSTOM_TRAIT_B, with a single trait of
         # CUSTOM_TRAIT_A
-        custom_a_rps = rp_obj.get_all_by_filters(
-            self.ctx,
-            filters={'required': ['CUSTOM_TRAIT_A', '!CUSTOM_TRAIT_B']})
-        self.assertEqual(1, len(custom_a_rps))
-        self.assertEqual(uuidsentinel.rp_uuid_1, custom_a_rps[0].uuid)
-        self.assertEqual('rp_name_1', custom_a_rps[0].name)
+        filters = {'required': ['CUSTOM_TRAIT_A', '!CUSTOM_TRAIT_B']}
+        expected_rps = ['rp_1']
+        custom_a_rps = self._run_get_all_by_filters(expected_rps,
+                                                    filters=filters)
+
+        self.assertEqual(uuidsentinel.rp_1, custom_a_rps[0].uuid)
         traits = trait_obj.get_all_by_resource_provider(
             self.ctx, custom_a_rps[0])
         self.assertEqual(1, len(traits))
@@ -1011,14 +898,14 @@ class TestResourceProviderAggregates(tb.PlacementDbBaseTestCase):
         )
 
         read_aggregate_uuids = rp.get_aggregates()
-        self.assertItemsEqual(aggregate_uuids, read_aggregate_uuids)
+        self.assertCountEqual(aggregate_uuids, read_aggregate_uuids)
 
         # Since get_aggregates always does a new query this is
         # mostly nonsense but is here for completeness.
         read_rp = rp_obj.ResourceProvider.get_by_uuid(
             self.ctx, uuidsentinel.rp_uuid)
         re_read_aggregate_uuids = read_rp.get_aggregates()
-        self.assertItemsEqual(aggregate_uuids, re_read_aggregate_uuids)
+        self.assertCountEqual(aggregate_uuids, re_read_aggregate_uuids)
 
     def test_set_aggregates_is_replace(self):
         start_aggregate_uuids = [uuidsentinel.agg_a, uuidsentinel.agg_b]
@@ -1029,7 +916,7 @@ class TestResourceProviderAggregates(tb.PlacementDbBaseTestCase):
         )
 
         read_aggregate_uuids = rp.get_aggregates()
-        self.assertItemsEqual(start_aggregate_uuids, read_aggregate_uuids)
+        self.assertCountEqual(start_aggregate_uuids, read_aggregate_uuids)
 
         rp.set_aggregates([uuidsentinel.agg_a])
         read_aggregate_uuids = rp.get_aggregates()
@@ -1087,6 +974,11 @@ class TestResourceProviderAggregates(tb.PlacementDbBaseTestCase):
             rp.set_aggregates(aggs)
             return rp
 
+        def _anchor(shr, anc):
+            return res_ctx.AnchorIds(
+                rp_id=shr.id, rp_uuid=shr.uuid,
+                anchor_id=anc.id, anchor_uuid=anc.uuid)
+
         # r1 and c1 constitute a tree.  The child is in agg1.  We use this to
         # show that, when we ask for anchors for s1 (a member of agg1), we get
         # the *root* of the tree, not the aggregate member itself (c1).
@@ -1120,49 +1012,43 @@ class TestResourceProviderAggregates(tb.PlacementDbBaseTestCase):
         # r2 via agg2 AND via agg3 through c2
         # r3 via agg3
         # s5 via agg1 and agg2
-        expected = set([(s1.uuid, rp.uuid) for rp in (s1, r1, r2, r3, s5)])
-        self.assertItemsEqual(
-            expected, rp_obj.anchors_for_sharing_providers(self.ctx, [s1.id]))
-
-        # Get same result (id format) when we set get_id=True
-        expected = set([(s1.id, rp.id) for rp in (s1, r1, r2, r3, s5)])
-        self.assertItemsEqual(
-            expected, rp_obj.anchors_for_sharing_providers(
-                self.ctx, [s1.id], get_id=True))
+        expected = set(_anchor(s1, rp) for rp in (s1, r1, r2, r3, s5))
+        self.assertCountEqual(
+            expected, res_ctx.anchors_for_sharing_providers(self.ctx, [s1.id]))
 
         # s2 gets s2 (self) and r3 via agg4
-        expected = set([(s2.uuid, rp.uuid) for rp in (s2, r3)])
-        self.assertItemsEqual(
-            expected, rp_obj.anchors_for_sharing_providers(self.ctx, [s2.id]))
+        expected = set(_anchor(s2, rp) for rp in (s2, r3))
+        self.assertCountEqual(
+            expected, res_ctx.anchors_for_sharing_providers(self.ctx, [s2.id]))
 
         # s3 gets self
         self.assertEqual(
-            set([(s3.uuid, s3.uuid)]), rp_obj.anchors_for_sharing_providers(
-                self.ctx, [s3.id]))
+            set([_anchor(s3, s3)]),
+            res_ctx.anchors_for_sharing_providers(self.ctx, [s3.id]))
 
         # s4 isn't really a sharing provider - gets nothing
         self.assertEqual(
-            set([]), rp_obj.anchors_for_sharing_providers(self.ctx, [s4.id]))
+            set([]), res_ctx.anchors_for_sharing_providers(self.ctx, [s4.id]))
 
         # s5 gets s5 (self),
         # r1 via agg1 through c1,
         # r2 via agg2
         # s1 via agg1 and agg2
-        expected = set([(s5.uuid, rp.uuid) for rp in (s5, r1, r2, s1)])
-        self.assertItemsEqual(
-            expected, rp_obj.anchors_for_sharing_providers(self.ctx, [s5.id]))
+        expected = set(_anchor(s5, rp) for rp in (s5, r1, r2, s1))
+        self.assertCountEqual(
+            expected, res_ctx.anchors_for_sharing_providers(self.ctx, [s5.id]))
 
         # validate that we can get them all at once
         expected = set(
-            [(s1.id, rp.id) for rp in (r1, r2, r3, s1, s5)] +
-            [(s2.id, rp.id) for rp in (r3, s2)] +
-            [(s3.id, rp.id) for rp in (s3,)] +
-            [(s5.id, rp.id) for rp in (r1, r2, s1, s5)]
+            [_anchor(s1, rp) for rp in (r1, r2, r3, s1, s5)] +
+            [_anchor(s2, rp) for rp in (r3, s2)] +
+            [_anchor(s3, rp) for rp in (s3,)] +
+            [_anchor(s5, rp) for rp in (r1, r2, s1, s5)]
         )
-        self.assertItemsEqual(
+        self.assertCountEqual(
             expected,
-            rp_obj.anchors_for_sharing_providers(
-                self.ctx, [s1.id, s2.id, s3.id, s4.id, s5.id], get_id=True))
+            res_ctx.anchors_for_sharing_providers(
+                self.ctx, [s1.id, s2.id, s3.id, s4.id, s5.id]))
 
 
 class SharedProviderTestCase(tb.PlacementDbBaseTestCase):
@@ -1207,31 +1093,47 @@ class SharedProviderTestCase(tb.PlacementDbBaseTestCase):
                              allocation_ratio=1.5)
             if cn is cn1:
                 tb.add_inventory(cn, orc.DISK_GB, 2000,
-                                 min_unit=10,
-                                 max_unit=100,
+                                 min_unit=100,
+                                 max_unit=2000,
                                  step_size=10)
 
         # Create the shared storage pool
-        ss = self._create_provider('shared storage')
+        ss1 = self._create_provider('shared storage 1')
+        ss2 = self._create_provider('shared storage 2')
 
         # Give the shared storage pool some inventory of DISK_GB
-        tb.add_inventory(ss, orc.DISK_GB, 2000,
-                         min_unit=10,
-                         max_unit=100,
-                         step_size=10)
-
-        # Mark the shared storage pool as having inventory shared among any
-        # provider associated via aggregate
-        tb.set_traits(ss, "MISC_SHARES_VIA_AGGREGATE")
+        for ss, disk_amount in ((ss1, 2000), (ss2, 1000)):
+            tb.add_inventory(ss, orc.DISK_GB, disk_amount,
+                             min_unit=100,
+                             max_unit=2000,
+                             step_size=10)
+            # Mark the shared storage pool as having inventory shared among
+            # any provider associated via aggregate
+            tb.set_traits(ss, "MISC_SHARES_VIA_AGGREGATE")
 
         # OK, now that has all been set up, let's verify that we get the ID of
-        # the shared storage pool when we ask for DISK_GB
-        got_ids = rp_obj.get_providers_with_shared_capacity(
-            self.ctx,
-            orc.STANDARDS.index(orc.DISK_GB),
-            100,
-        )
-        self.assertEqual([ss.id], got_ids)
+        # the shared storage pool
+        got_ids = res_ctx.get_sharing_providers(self.ctx)
+        self.assertEqual(set([ss1.id, ss2.id]), got_ids)
+
+        request = placement_lib.RequestGroup(
+            use_same_provider=False,
+            resources={orc.VCPU: 2,
+                       orc.MEMORY_MB: 256,
+                       orc.DISK_GB: 1500})
+        has_trees = res_ctx._has_provider_trees(self.ctx)
+        sharing = res_ctx.get_sharing_providers(self.ctx)
+        rg_ctx = res_ctx.RequestGroupSearchContext(
+            self.ctx, request, has_trees, sharing)
+
+        VCPU_ID = orc.STANDARDS.index(orc.VCPU)
+        DISK_GB_ID = orc.STANDARDS.index(orc.DISK_GB)
+
+        rps_sharing_vcpu = rg_ctx.get_rps_with_shared_capacity(VCPU_ID)
+        self.assertEqual(set(), rps_sharing_vcpu)
+
+        rps_sharing_dist = rg_ctx.get_rps_with_shared_capacity(DISK_GB_ID)
+        self.assertEqual(set([ss1.id]), rps_sharing_dist)
 
 
 # We don't want to waste time sleeping in these tests. It would add
